// services/games/turtlesoup.ts
import { Result, createSuccess, createError } from '@/types/result'
import { cloudApi } from '@/api'
import { USE_MOCK } from '@/services'
import { 
  type TurtleSoupGame, 
  type TurtleSoupQuestion, 
  type TurtleSoupGameResult,
  type TurtleSoupGameStatus,
  type TurtleSoupDifficulty,
  type CreateTurtleSoupGameParams
} from '@/types/games/turtlesoup'

// Mock数据
const mockGame: TurtleSoupGame = {
  id: 'mock-ts-1',
  title: '死亡的摄影师',
  description: '一个摄影师在自己的工作室里死亡，周围没有任何人，死因是什么？',
  storyteller: 'user-123',
  status: 'active',
  solution: '这个摄影师在深海拍摄鲨鱼时，使用了闪光灯，引来了鲨鱼攻击并死亡。',
  difficulty: 'medium',
  createTime: new Date().toISOString(),
  hints: [
    { content: '死亡地点不是工作室', revealed: false },
    { content: '摄影师当时正在进行水下摄影', revealed: false },
    { content: '死亡与水中的生物有关', revealed: false }
  ],
  questions: [],
  players: [
    { id: 'user-456', name: '玩家1', avatar: 'https://example.com/avatar1.png', joinedAt: new Date().toISOString() },
    { id: 'user-789', name: '玩家2', avatar: 'https://example.com/avatar2.png', joinedAt: new Date().toISOString() }
  ]
}

// 海龟汤游戏服务
export const turtleSoupService = {
  /**
   * 获取游戏数据
   * @param gameId 游戏ID
   * @param role 用户角色 (storyteller/player)
   */
  async getGameData(gameId: string, role: string): Promise<Result<TurtleSoupGame>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          // 根据角色调整返回的数据
          if (role === 'storyteller') {
            // 主持人可以看到所有信息
            resolve(createSuccess({
              ...mockGame,
              hints: mockGame.hints.map(h => ({ ...h, revealed: true }))
            }));
          } else {
            // 普通玩家只能看到已揭示的提示
            resolve(createSuccess(mockGame));
          }
        }, 300);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ game: TurtleSoupGame }>('getTurtleSoupGame', { gameId, role })
      .then(result => {
        if (result.success && result.data && result.data.game) {
          // 确保返回的是TurtleSoupGame对象
          return createSuccess(result.data.game);
        }
        
        // 如果云函数调用成功但没有返回游戏数据
        return createError('未找到游戏数据');
      })
      .catch(error => {
        console.error('获取海龟汤游戏数据失败:', error);
        return createError(error.message || '获取游戏数据失败');
      });
  },

  /**
   * 提交问题
   * @param gameId 游戏ID
   * @param content 问题内容
   */
  async submitQuestion(gameId: string, content: string): Promise<Result<{ questionId: string }>> {
    if (!content.trim()) {
      return createError('问题内容不能为空');
    }
    
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          const questionId = `q-${Date.now()}`;
          resolve(createSuccess({ questionId }));
        }, 300);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ questionId: string }>('submitTurtleSoupQuestion', { gameId, content });
  },

  /**
   * 回答问题
   * @param questionId 问题ID
   * @param answer 答案 ('yes', 'no', 'irrelevant')
   */
  async answerQuestion(questionId: string, answer: string): Promise<Result<{ success: boolean }>> {
    const validAnswers = ['yes', 'no', 'irrelevant'];
    
    if (!validAnswers.includes(answer)) {
      return createError('无效的答案，只能是"yes"、"no"或"irrelevant"');
    }
    
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess({ success: true }));
        }, 200);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ success: boolean }>('answerTurtleSoupQuestion', { questionId, answer });
  },

  /**
   * 公开提示
   * @param gameId 游戏ID
   * @param hintIndex 提示索引
   */
  async revealHint(gameId: string, hintIndex: number): Promise<Result<{ success: boolean }>> {
    if (hintIndex < 0) {
      return createError('提示索引无效');
    }
    
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          // 检查提示索引是否有效
          if (hintIndex >= mockGame.hints.length) {
            resolve(createError('提示索引超出范围'));
            return;
          }
          
          resolve(createSuccess({ success: true }));
        }, 200);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ success: boolean }>('revealTurtleSoupHint', { gameId, hintIndex });
  },

  /**
   * 结束游戏
   * @param gameId 游戏ID
   * @param result 游戏结果对象
   */
  async endGame(gameId: string, result: { solved: boolean; solvedBy?: string }): Promise<Result<TurtleSoupGameResult>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          const gameResult: TurtleSoupGameResult = {
            gameId,
            solved: result.solved,
            solvedBy: result.solvedBy,
            duration: 1800, // 30分钟
            questionCount: 15,
            hintsRevealed: 2,
            completionTime: new Date().toISOString()
          };
          
          resolve(createSuccess(gameResult));
        }, 400);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ gameResult: TurtleSoupGameResult }>('endTurtleSoupGame', { gameId, result })
      .then(response => {
        if (response.success && response.data && response.data.gameResult) {
          return createSuccess(response.data.gameResult);
        }
        
        return createError(response.message || '结束游戏失败');
      });
  },
  
  /**
   * 创建新游戏
   * @param gameData 游戏数据
   */
  async createGame(gameData: CreateTurtleSoupGameParams): Promise<Result<{ gameId: string }>> {
    // 数据验证
    if (!gameData.title.trim()) return createError('游戏标题不能为空');
    if (!gameData.description.trim()) return createError('游戏描述不能为空');
    if (!gameData.solution.trim()) return createError('游戏答案不能为空');
    if (!gameData.hints || gameData.hints.length === 0) return createError('至少需要提供一个提示');
    
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          const gameId = `game-${Date.now()}`;
          resolve(createSuccess({ gameId }));
        }, 500);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ gameId: string }>('createTurtleSoupGame', gameData);
  },
  
  /**
   * 获取用户创建的游戏列表
   */
  async getCreatedGames(): Promise<Result<TurtleSoupGame[]>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess([mockGame]));
        }, 400);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ games: TurtleSoupGame[] }>('getCreatedTurtleSoupGames')
      .then(result => {
        if (result.success && result.data && result.data.games) {
          return createSuccess(result.data.games);
        }
        
        if (result.success) {
          return createSuccess([]);
        }
        
        return createError(result.message || '获取游戏列表失败');
      });
  },
  
  /**
   * 获取用户参与的游戏列表
   */
  async getJoinedGames(): Promise<Result<TurtleSoupGame[]>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess([{...mockGame, storyteller: 'other-user'}]));
        }, 400);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ games: TurtleSoupGame[] }>('getJoinedTurtleSoupGames')
      .then(result => {
        if (result.success && result.data && result.data.games) {
          return createSuccess(result.data.games);
        }
        
        if (result.success) {
          return createSuccess([]);
        }
        
        return createError(result.message || '获取游戏列表失败');
      });
  },
  
  /**
   * 提交游戏解答
   * @param gameId 游戏ID
   * @param solution 玩家提交的解答
   */
  async submitSolution(gameId: string, solution: string): Promise<Result<{ correct: boolean }>> {
    if (!solution.trim()) {
      return createError('解答内容不能为空');
    }
    
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          // 模拟50%的概率答对
          const correct = Math.random() > 0.5;
          resolve(createSuccess({ correct }));
        }, 300);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ correct: boolean }>('submitTurtleSoupSolution', { gameId, solution });
  },
  
  /**
   * 加入游戏
   * @param gameId 游戏ID
   * @param code 游戏邀请码(私人游戏需要)
   */
  async joinGame(gameId: string, code?: string): Promise<Result<{ success: boolean }>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess({ success: true }));
        }, 300);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ success: boolean }>('joinTurtleSoupGame', { gameId, code });
  },
  
  /**
   * 离开游戏
   * @param gameId 游戏ID
   */
  async leaveGame(gameId: string): Promise<Result<{ success: boolean }>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess({ success: true }));
        }, 300);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ success: boolean }>('leaveTurtleSoupGame', { gameId });
  },
  
  /**
   * 更新游戏状态
   * @param gameId 游戏ID
   * @param status 新状态
   */
  async updateGameStatus(gameId: string, status: TurtleSoupGameStatus): Promise<Result<{ success: boolean }>> {
    // 如果在开发环境或非小程序环境，使用Mock数据
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => {
        setTimeout(() => {
          resolve(createSuccess({ success: true }));
        }, 300);
      });
    }
    
    // 使用cloudApi调用云函数
    return cloudApi.call<{ success: boolean }>('updateTurtleSoupGameStatus', { gameId, status });
  }
};