// types/games/turtlesoup.ts

/**
 * 海龟汤游戏状态
 */
export type TurtleSoupGameStatus = 'created' | 'active' | 'completed' | 'abandoned';

/**
 * 海龟汤游戏难度
 */
export type TurtleSoupDifficulty = 'easy' | 'medium' | 'hard' | 'expert';

/**
 * 海龟汤问题答案类型
 */
export type TurtleSoupAnswerType = 'yes' | 'no' | 'irrelevant';

/**
 * 游戏提示结构
 */
export interface TurtleSoupHint {
  content: string;     // 提示内容
  revealed: boolean;   // 是否已公开
  revealedAt?: string; // 公开时间
}

/**
 * 海龟汤问题
 */
export interface TurtleSoupQuestion {
  id: string;                   // 问题ID
  content: string;              // 问题内容
  answer?: TurtleSoupAnswerType;// 回答类型：是/否/不重要
  askedBy: string;              // 提问者ID
  askedByName?: string;         // 提问者名称
  timestamp: string;            // 提问时间(ISO格式)
  answered: boolean;            // 是否已回答
  answeredAt?: string;          // 回答时间
}

/**
 * 游戏参与者结构
 */
export interface TurtleSoupPlayer {
  id: string;          // 用户ID
  name: string;        // 用户名称
  avatar?: string;     // 头像
  joinedAt: string;    // 加入时间
  role?: 'player' | 'storyteller'; // 角色：玩家/讲述者
}

/**
 * 海龟汤游戏
 */
export interface TurtleSoupGame {
  id: string;                        // 游戏ID
  title: string;                     // 游戏标题
  description: string;               // 游戏描述/故事
  solution: string;                  // 解决方案
  hints: TurtleSoupHint[];           // 提示列表
  questions: TurtleSoupQuestion[];   // 问题列表
  status: TurtleSoupGameStatus;      // 游戏状态
  createTime: string;                // 创建时间(ISO格式)
  startTime?: string;                // 开始时间
  endTime?: string;                  // 结束时间
  storyteller: string;               // 讲述者ID
  storytellerName?: string;          // 讲述者名称 
  players: TurtleSoupPlayer[];       // 参与者列表
  difficulty: TurtleSoupDifficulty;  // 游戏难度
  maxPlayers?: number;               // 最大玩家数
  isPrivate?: boolean;               // 是否为私人游戏
  code?: string;                     // 加入码（私人游戏时使用）
  tags?: string[];                   // 标签
  likeCount?: number;                // 点赞数
  playCount?: number;                // 游玩次数
  solvedCount?: number;              // 被解决次数
}

/**
 * 海龟汤游戏结果
 */
export interface TurtleSoupGameResult {
  gameId: string;                    // 游戏ID
  solved: boolean;                   // 是否解决
  solvedBy?: string;                 // 解决者ID
  solvedByName?: string;             // 解决者名称
  solution?: string;                 // 提交的解答
  duration: number;                  // 游戏时长(秒)
  questionCount: number;             // 提问次数
  hintsRevealed: number;             // 使用的提示数
  completionTime: string;            // 完成时间(ISO格式)
  satisfaction?: number;             // 满意度评分(1-5)
  feedback?: string;                 // 反馈
}

/**
 * 创建海龟汤游戏参数
 */
export interface CreateTurtleSoupGameParams {
  title: string;                     // 游戏标题
  description: string;               // 游戏描述/故事
  solution: string;                  // 解决方案
  hints: string[];                   // 提示列表
  difficulty: TurtleSoupDifficulty;  // 游戏难度
  isPrivate?: boolean;               // 是否为私人游戏
  tags?: string[];                   // 标签
}

/**
 * 游戏统计信息
 */
export interface TurtleSoupGameStats {
  totalGames: number;                // 总游戏数
  completedGames: number;            // 已完成游戏数
  averageDuration: number;           // 平均游戏时长(秒)
  averageQuestions: number;          // 平均提问数
  solveRate: number;                 // 解决率(0-1)
  mostUsedHints: number;             // 最常使用的提示次数
  difficultyDistribution: {          // 难度分布
    easy: number;
    medium: number;
    hard: number;
    expert: number;
  };
}